(***************************************************************************)
(**                            UNIT BOXVARS.PAS                           **)
(***************************************************************************)
(**  Variablen/Typen/Konstanten/etc-Deklarationen fr die Mailbox.        **)
(**                                                                       **)
(** Diese Unit ist Teil des MCUT-Sourcecodes, alle Rechte liegen bei      **)
(** DG4IAD und DG6VJ                                                      **)
(**                                                                       **)
(** dg6vj, mar 96                                                         **)
(***************************************************************************)

{ SID begrenzt auf 80 Zeichen }

UNIT BOXVARS;


INTERFACE
USES FileHash;

CONST UserFileVersion : STRING[8] = 'MCUT-005';
      LanguageFileVersion = 6;  {Version der aktuellen Sprachen-Files}
      MaxBoxFrameSize = 5000;   {Wieviele Bytes werden maximal beim
                                 READ-Befehl ausgesendet, wenn in der Send-
                                 Queue noch Eintrge sind und nix mehr
                                 zum senden im MCUT-Buffer steht}
      THEBOX_SID = '[THEBOX-1.9-H$]';   {SID der Box, wenn in BOX.SF Spalte
                                 33 '2' angegeben ist. (Fr MYBBS-Forwarding
                                 an BayCom und DieBox-Systeme)}
      CRLF : ARRAY[1..2] OF CHAR = (#13,#10);
      AutorouterAlternUm = 600; {Um wieviele SEKUNDEN soll die schnellste
                                 Zeit im Autorouter gealtert werden, wenn
                                 die keine neue schnellste Zeit errechnet
                                 wurde?}

      VERWALT_DEL = 1;
      VERWALT_READ = 2;
      VERWALT_HOLD = 4;
      VERWALT_PMAIL = 8;

      SFLISTE_DEL = 1;
      SFLISTE_LTMIT = 2;
      SFLISTE_HOLD = 4;

      CHECK_DEL = 1;
      CHECK_HOLD = 2;
      CHECK_SYSOP = 4;

(*
 * PROFILE.BOX Konstanten (iad, dez 95)
 *)
      {FLAGS fr PROFILE.BOX (was wird geloggt, was nicht)}
      PBOX_SFCRC = 1;
      PBOX_USERPWFAIL = 2;
      PBOX_SFPWFAIL = 4;
      PBOX_ROUTINGERROR = 8;
      PBOX_HOLD = 16;
      PBOX_BADCALL = 32;
      PBOX_REDIRECT = 64;
      PBOX_IOERROR=128;
      PBOX_DEBUG=256;

      {als default wird alles geloggt:}
      WHAT_TO_LOG:LONGINT=$FFFFFFF;


TYPE str3 = STRING[3];
     str5 = STRING[5];
     str6 = STRING[6];
     str8 = STRING[8];
     str10 = STRING[10];
     str15 = STRING[15];
     str40 = STRING[40];
     str127 = STRING[127];

     tArr255 = ARRAY[1..255] OF CHAR;


(**************************************************************************)
(*         Record fr CHECK-Ausgabe im Hintergrund (iad,dez 95)           *)
(*                     Gendert: dg6vj, mar 96                            *)
(**************************************************************************)

CONST
  (*
   *  Flags :
   *  BIT 1 : nur eintrge, die im HOLD stehen.
   *  BIT 2 : nur eintrge mit datum > lastlogin.
   *  BIT 3 : auch gelschte eintrge.
   *  BIT 4 : nur gelschte eintrge
   *)
  CHK_HOLD = 1;
  CHK_NEU  = 2;
  CHK_DEL  = 4;
  CH_DELONLY = 8;
  (*
   * Wieviele Bytes werden jeweils auf einmal ausgeben.
   * (vernnftiger Wert: 8 Frames a 256 Bytes)
   *)
  CHK_MAXBLOCK = 2048;
TYPE

  pCheckAusgabe  = ^tCheckAusgabe;
  tCheckAusgabe  = RECORD
      start,stop,aktuell:LONGINT;
      info:STRING[80];
      notboards:STRING;
      flags:BYTE;
      anz: WORD;
  END;

(**************************************************************************)
(*         Record fr LIST-Ausgabe im Hintergrund (iad,dez 95)           *)
(**************************************************************************)

CONST
  (*
   *  Flags :
   *)
  LST_HOLD = 1;
  LST_NEU  = 2;
  LST_DEL  = 4;
  (*
   * Wieviele Bytes werden jeweils auf einmal ausgeben.
   * (vernnftiger Wert: 8 Frames a 256 Bytes)
   *)
  LST_MAXBLOCK = 2048;

TYPE

  pListAusgabe  = ^tListAusgabe;
  tListAusgabe  = RECORD
      start,stop,aktuell:LONGINT;
      rubrik:STR8;
      info:STRING[80];
      flags:BYTE;
      Anz: WORD;
      Lock: BYTE;
      OK: BOOLEAN;
  END;


(********************* SFLISTE im Speicher ********************************)
(* iad, jun '96 *)

     pSFList = ^TSFList;
     tSFList = RECORD
        BID:STRING[12];
        Pos_In_SFliste:WORD;
        size:LONGINT;
        semaphore:BYTE;
        next:pSFList;
     END;



{********************** BOX.SF im Speicher *******************************}
     pBoxSF = ^tBoxSF;
     tBoxSF = RECORD
       next: pBoxSF;
       Call: STRING[6];         {Rufzeichen der Gegenstation}
       Times: STRING[24];       {Zu welchen Zeiten wird was angestoen?}
       SID: BYTE;               {Welche SID soll benutzt werden?}
       Pfad: STRING[100];       {Connect-Pfad}
       WhatToForward: STRING;   {Was soll geforwardet werden? Hier wird
                                 einfach der What_To_Forward - Teil nach
                                 dem Call hingeschrieben}
       AnzMails,AnzImRam:WORD;
       LastConnect:LONGINT;
       First_SFLISTE : WORD;
       Next_SFLISTE :WORD;
     END;



{******************** INDEX - Files **************************************}
     Verwalt_Rec = RECORD        {Hier werden die Infos der einzelnen}
       Titel: STRING[80];        {Rubriken abgelegt.}
       Absender: STRING[6];
       Rubrik: STRING[8];        {Rubrik/Empfangs-Call}
       MBX: STRING[8];           {Verteiler oder Mailbox-Adresse}
       FileName: STRING[30];     {Filename (ohne BoxPath!)}
       Rx_Datum: LONGINT;        {Empfangs Datum und Zeit - gepackt}
       BID: STRING[12];          {Bulletin-ID der Mail}
       Groesse: LONGINT;         {Groesse des Files}
       Lifetime: WORD;           {Lifetime der Mail}
       Flags: WORD;              {Flags:
                                   Bit 1: Gelscht-Flag
                                   Bit 2: Read-Flag
                                   Bit 3: Hold-Flag
                                   DG4IAD:
                                   Bit 4: ist P-Mail, fr eintrag in Checkliste}
     END;



{************** Liste von Mails, die noch gelesen werden mssen **********}
{****************************** (Send-Queue) *****************************}
{Konstanten frs Aussehen der Mailausgabe}
CONST
     SET_Bytes = 1;
     SET_Datum = 2;
     SET_R_Vollst = 4;
     SET_Lifetime = 8;
     SET_R_Kurz = 16;
     SET_Read = 32;
     SET_Sender = 64;
     SET_Uhrzeit= 128;

TYPE pBoxRead = ^TBoxRead;
     TBoxRead = RECORD
       Next: pBoxRead;          {Pointer auf den nchsten Eintrag der Liste}
       IdxInfo: Verwalt_Rec;    {Eintrag aus dem Index-File}
       LEN: LONGINT;            {Wieviele Bytes (ohne CR) wurden schon
                                 gesendet?}
       Started: BOOLEAN;        {True, wenn schon etwas von diesem File
                                 gesendet wurde}
       Set_Info: BYTE;          {Wie sollen die ausgegebenen Mails aussehen?
                                 Bitfeld: Bit 0: Bytes
                                              1: Datum
                                              2: R_Vollst
                                              3: Lifetime
                                              4: R_Kurz
                                              5: Read
                                              6: Sender
                                              7: Uhrzeit}
       BinOffset: LONGINT;
       Kompr: BOOLEAN;
     END;



{*********************** COMMAND-QUEUE *************************************}
     pBoxCmdQueue = ^tBoxCmdQueue;
     tBoxCmdQueue = RECORD
       Befehl: STRING;
       next: pBoxCmdQueue;
     END;

{***************** REJECT.BOX-Abbild im Speicher ***************************}
     pReject_Box = ^tReject_Box;
     tReject_Box = RECORD
       next: pReject_Box;
       Typ: CHAR;
       Was: BYTE;               {1 = Reject
                                 2 = Hold
                                 3 = Local Hold}
       Absender: STRING[6];
       Rubrik: STRING[8];
       MBX: STRING[8];
       BID: STRING[12];
       Not_Hold: BYTE;          {Welche Files sind nicht betroffen?
                                 Bit 0: Mit Passwort eingespielte Files (1)
                                 Bit 1: Usermails (2)}
     END;


{*********************** Passwort-Abfrage beim login ***********************}
     tPwData = RECORD
       PwStrg: str5;
       OldStatus: BYTE;
     END;


{*********************** Daten der User in der Box *************************}
     tBoxReadBIN = RECORD       {Frs Lesen von BIN-Mails}
       f: Text;
       BIN_F: File;
       BIN: BOOLEAN;
       BIN_CRC_Got: WORD;
       BIN_CRC: WORD;
     END;


     pSendData = ^tSendData;
     TSendData = RECORD         {Wird zum Verschicken von Mails bentigt}
       BID: STRING[12];         {BID von abschickenden Mails}
       LT: WORD;                {Lifetime}
       LTOK: BOOLEAN;           {Wurde eine korrekte Lifetime gefunden?}
       An: STRING[8];           {Empfnger der Mail}
       An_Trans: STRING[8];
       Verteiler: STRING[8];    {Verteiler der Mail}
       UserMail: BOOLEAN;       {TRUE, wenn die Mail eine Usermail ist}
       Titel: STRING[80];       {Titel der Mail}
       Hierach: STRING[128];    {Hierachie der Empfnger-Box}
       FileName: STRING[12];    {Dateiname des Files (OHNE(!) Directory}
       f: Text;                 {File, in dem der eigentliche Text
                                 gespeichert wird}
       BIN: BOOLEAN;            {Wird grade ein Binr-File empfangen?}
       BIN_Len: LONGINT;        {Lnge des Files}
       BIN_Akt: LONGINT;        {Stelle, wo wir im Augenblick im File sind}
       BIN_F: FILE;             {File, in dem Binr-Mails gespeichert
                                 werden}
       BIN_CRC: WORD;           {16Bit-CRC (selbst berechnet)}
       BIN_CRC_Got: WORD        {empfangene CRC}
     END;


     TBoxData=RECORD
       oldStatus: INTEGER;      {Status, bevor die Box gestartet wurde.
                                 (Damit beim Quit die Box nicht verlassen
                                 wird) -1 = Direkt in die Box 'rein}
       Proto: BOOLEAN;          {Alle Eingaben protokollieren? (Filename
                                 CALL.PRO im Verzeichnis PROTO)}
       ProtoFile: Text;         {Textfile, in dem Protokolliert wird}
       status: BYTE;
       Prompt: BYTE;
       Prompt_Str: ^str40;      {Prompt-String, wenn Prompt=0 ist}
       Rubrik: STRING[8];       {Aktuelle Rubrik}
       M_Sender: STRING[6];     {Absender der zuletzt gelesenen Mail
                                 (fr den REPLY-Befehl}
       M_Titel: STRING[80];     {Titel der zuletzt gelesenen Mail
                                 (REPLY/COMMENT-Befehl}
       M_Verteil: STRING[8];    {Verteiler der zuletzt gelesenen Mail
                                 (COMMENT-Befehl)}
       M_Nr   :  WORD;          {Zuletzt bearbeitete Mail, fr NEXT-Befehl}
       {$IFDEF EDITOR}
       M_Filename: STRING[30];  {Dateiname der zuletzt gelesenen Mail}
       {$ENDIF}
       MYBBS: STRING[6];        {Das MYBBS der Station}
       SendData: pSendData;     {Beim Abschicken von Mails}
       Start_SF: BOOLEAN;       {TRUE, wenn eingegebenes Rufzeichen ein
                                 S&F-Partner ist, also beim Connect
                                 die Anzahl der Connects nicht hoch
                                 gezhlt werden}
       Spk: STRING[3];          {Eingestellte Systemsprache}
       LoginTime: LONGINT;      {Wann hat diese Station sich eingeloggt?}
       LastSendTime: LONGINT;   {Und wann wurde die letzte Zeile dieser
                                 Station empfangen?}
       FirstBoxRead: pBoxRead;  {Adresse des ersten Eintrags in der
                                 Send-Queue}
       CmdQueue: pBoxCmdQueue;  {Adresse des ersten Eintrags der Kommand-
                                queue}
       HelpLevel: BYTE;         {Welcher Helplevel ist eingestellt?
                                  0 = Experte
                                  1 = ein lngerer cText wird ausgegeben
                                  2 = nur ein kurzer cText wird ausgegeben}
       PW: BOOLEAN;              {Der User hat ein Passwort eingestellt.}
       SF_PW: BOOLEAN;           {Es wird ein Passwort frs S&F bentigt}
       SF_PW_What: STRING[5];    {Errechnetes Passwort vom S&F}
       CheckAusgabe: pCheckAusgabe;     {Checkfile im Hintergrund ausgeben}
       ListAusgabe: pListAusgabe;       {Rubriken-Listen-Ausgabe im
                                  Hintergrund}
     END;


{********************************** User-Daten *****************************}
     R_UserFile = RECORD         {Hier werden die User-Infos der Box abgelegt}
       Call: STRING[6];          {Rufzeichen der Gegenstation}
       Name: STRING[20];         {Name der Gegenstation}
       MYBBS: STRING[6];         {Heimatmailbox}
       Prompt: BYTE;             {Promptart:   1 = DieBox
                                             255 = Eigen}
       PromptStr: STRING[40];    {Wenn Prompt=255 steht hier der eigene Pr.}
       LastLogin: LONGINT;       {Zeitpunkt des letzten Logins - mit PackTime
                                  gepackt}
       LastQuit: LONGINT;        {Wann wurde das letzte Mal QUIT gemacht?}
       LastMYBBS: LONGINT;       {Wann wurde da letzte Mal MYBBS gesetzt?
                                  (auch per S&F!) (PackTime)}
       Spk: STRING[3];           {Eingestellte Systemsprache}
       Anz_Send: WORD;           {Anzahl der gesendeten Messages}
       Anz_Rec: WORD;            {Anzahl der empfangenen (persnlichen) MSGs}
       Anz_Read: WORD;           {Anzahl gelesener Messages}
       Anz_Logins: WORD;         {Anzahl durchgefhrter Logins}
       ReadLock: BYTE;           {Readlock gesetzt/nicht gesetzt?
                                   0 = nicht gesetzt
                                   1 = gelesene Mails werden freigegeben
                                   2 = gar keine privaten Mails knnen
                                       gelesen werden}
       ReadList: WORD;           {Gesetzte/nicht gesetzte Bytes fr den
                                  Read-Befehl

                                  Bit 0: Byteanzahl Ausgeben
                                  Bit 1: Datum ausgeben
                                  Bit 2: R: - Header vollstndig ausgeben
                                  Bit 3: Lifetime ausgeben
                                  Bit 4: R: - Header in Kurzform ausgeben
                                  Bit 5: Rufzeichen ausgeben, die die
                                         Nachricht gelesen haben
                                  Bit 6: Herkunftsbox ausgeben (sender)
                                  Bit 7: Uhrzeit ausgeben
                                 }
       HelpLevel: BYTE;         {Welcher Helplevel ist eingestellt?
                                  0 = Experte
                                  1 = kein lngerer cText wird ausgegeben
                                  2 = der lange cText wird ausgegeben}
       Update_Type: CHAR;       {Fr WP-Updates:
                                   U: Daten wurden vom User selbst eingegeben
                                      -> mit grter wahrscheinlichkeit
                                      korrekt
                                   G: Daten wurden aus dem Header von Mails
                                      gewonnen -> nicht sicher korrekt
                                   I: Infos ber forwardende Mailboxen}
       ZIP: LONGINT;            {ZIP-Code des Users. Wird von MCUT nicht
                                 verwendet (nur fr WP-Updates)
                                 (unbekannt = -1)}
       QTH: STRING[20];         {QTH des Users. Wird von MCUT nicht benutzt.
                                 (nur fr WP-Updates)}
       Flags: BYTE;             {Sonstige Flags.
                                  Bit 0: Eintrag seit dem letzten WP-Update
                                         verndert}
       UserPassWord : STRING[80];
       Friends      : STRING[60]; {Liste von Calls, getrennt durch space,
                                   die trotz ReadLock lesen drfen}
       Level        : BYTE;
       reserved_for_future: ARRAY[1..450] OF BYTE;
     END;


{******************* OwnDat - Daten der eigenen Mailbox ********************}
     T_OwnBoxData = RECORD
       SF_Call: STRING[9];      {Rufzeichen, das fr S&F benutzt wird}
       Hierach: STRING[30];     {Hierachie der Mailbox}
       SFInfo: STRING[50];      {Info in eckigen klammern im S&F-Header}
       SF_Proto: BOOLEAN;       {Soll der S&F-Verkehr protokolliert
                                 werden? (kann ein groes File ergeben)}
       User_LT: WORD;           {Lifetime fr Usermails}
       BBS_Connect: BOOLEAN;    {Darf die Mailbox connected werden?}
       Header_Eintrag: BOOLEAN; {Soll sich die eigene Mailbox in den
                                 S&F-Header mit eintragen? Sollte
                                 normalerweise IMMER auf TRUE stehen}
       Beacon: BOOLEAN;         {Mail-Bake aussenden? (J/N)}
       M_E_LT: WORD;            {Lifetime fr M und E-Mails}
       M_E_MBX: STRING[8];      {Verteiler fr M und E-Mails}
       M_E_Use: BOOLEAN;        {Sollen M/E-Files verschickt werden?}
       DB0_MYBBS: BOOLEAN;      {Sollen auch DB0-Mailboxen als MYBBS
                                 angenommen werden?}
       SF_Ab_kBytes: LONGINT;   {Wenn der plattenspeicher unter die hier
                                 angegebene Gre fllt, ist kein s&f mehr
                                 mglich.}
       SF_Mit_jedem: BOOLEAN;   {Auch mit Stationen, die nicht in BOX.SF
                                 stehen, S&F machen?}
       M_FILTER: STRING;        {Filename vom M_FILTER (leer, wenn nicht
                                 vorhanden)}
       Third_Party: BOOLEAN;    {Third-Party-Mail enablen?}
       System_Msg_to: STRING[20];
     END;


{******************************** DIR - Info ********************************}
     R_DirInfo = RECORD
       Rubrik: STRING[8];        {Name der Rubrik (auch Userfiles)}
       Mails: WORD;              {Anzahl Mails in dieser Rubrik}
       Art: CHAR;                {Info- oder User-Rubrik}
     END;


{*********************** S&F-Liste (SFLISTE.) ****************************}
     SF_File_Rec = RECORD        {Aufbau des S&F-Files (SFLISTE)}
       SF_Partner: STRING[6];    {Call der Station, fr die dieser
                                  Eintrag bestimmt ist}
       Von: STRING[6];           {Von wem ist die Mail?}
       An: STRING[8];            {An wen geht sie? (User/Rubrik)}
       Via: STRING[8];           {An@Via (Verteiler/Boxcall)}
       Hierachie: STRING[50];    {Wenn BoxCall dann wird hier die
                                  Hierachie eingetragen}
       LT: WORD;                 {Lifetime der Mail}
       Titel: STRING[80];        {Titel der Mail}
       FileName: STRING[30];     {Dateiname der Mail (ohne Verzeichnis)}
       Prio: BYTE;               {Nach diesem Feld kann das File sortiert
                                  werden, so da beim Forward z.B. die F-
                                  Mails bevorzugt geforwardet werden}
       Art: CHAR;                {(P)ersnliche, (B)ulletin oder (G)roups
                                 M = MYBBS-Forwarding}
       BID: STRING[12];          {BID oder MID}
       Groesse: LONGINT;         {Gre des Files (fr FBB-S&F)}
       Flags: WORD;              {Flags:
                                    Bit 0: Gelscht-Flag
                                    Bit 1: Lifetime vom Absender mitgegeben?
                                    Bit 2: File fr Hold markiert?}
     END;


(******************************* CHECK-File *******************************)
     CheckRec = RECORD
       Von: STRING[6];          {Von wem is die Mail (Rufzeichen)}
       An: STRING[8];           {An welche Rubrik geht die Mail}
       Nr: WORD;                {Welche Nummer hat sie in dieser Rubrik}
       RX_Zeit: LONGINT;        {Datum und Uhrzeit, an der die Mail
                                 gekommen ist}
       MBX: STRING[6];          {Verteiler der Mail}
       Bytes: LONGINT;          {Gre in Bytes}
       LT: WORD;                {Lifetime der Mail}
       Titel: STRING[25];       {Titel der Mail - 25 Zeichen lang, da nicht
                                 mehr dargestellt werden kann}
       BID: STRING[12];         {BID der Mail (zum lschen)}
       Flags: WORD;             {Flags:
                                    Bit 0: Delete-Flag
                                    Bit 1: Hold
                                    Bit 2: Sysop Only}
     END;


{***************************** AUTOROUTER *********************************}
     Stn_Best_Rec = RECORD       {Record mit den besten Zeiten - als ARRAY
                                  [1..3] in Stn_Rec verwendet}
       From: STRING[6];          {Call der Station}
       Time: LONGINT;            {Zeit in Sekunden}
     END;

     Stn_Rec = RECORD            {In diesem Record werden die Infos fr den}
                                 {Autorouter gespeichert.}
       Call: STRING[8];          {Logo, das Rufzeichen}
       HPATH: STRING[40];        {Auch Logo, die Hierachie dieses Calls}
       RXFrom: STRING[6];        {Call des Nachbarn, mit dem CALL s&f macht}
       Desc: STRING[40];         {Beschreibung der Station (aus [])}
       LastTX: LONGINT;          {Datum der letzten Mail, die von dieser
                                  Station gesichtet wurde. (Packtime)}
       Best: ARRAY[1..3] OF Stn_Best_Rec;   {Array mit den 3 besten Forward-
                                  wegen. Array deshalb, weil dieses besser
                                  sortiert werden kann. (Bubble Sort).
                                  ALTERN NICHT VERGESSEN(!!!!)}
       MSGCt: LONGINT;           {Wie viele Mails sind von/ber diese Station
                                  gekommen/gegangen?}
     END;


(***************************************************************************)
(**                      Mailbox Online-Hilfe                             **)
(***************************************************************************)
TYPE pOnlineHelp = ^tOnlineHelp;
     tOnlineHelp = RECORD
       next: pOnlineHelp;       {Zeiger auf das nchste Listenelement}
       Command: STRING[20];     {Eigentlicher Befehl}
       Filename: STRING[8];     {Dateiname der Hilfedatei in BoxPath+'\HELP'}
       Level: BYTE;             {Ab welchem Level ist der Zugriff auf diesen
                                 Befehl erlaubt?}
     END;


(***************************************************************************)
(**                           RECORDS fr S&F                             **)
(***************************************************************************)

  SF_Features = RECORD
    A: BOOLEAN;                 {ACK-Mails}
    M: BOOLEAN;                 {Mail-ID (BID fr User-Mails)}
    H: BOOLEAN;                 {Hierachisches Forward}
    G: BOOLEAN;                 {Gruppen-Forward}
    D: BOOLEAN;                 {FBB-Forward nach deutschen Specs}
    B: BOOLEAN;                 {Gepackter FBB-Forward}
    B_Lev: BYTE;                {Level der Implementierung - dies ist die
                                 Nummer hinter dem Buchstaben, z.B.
                                 [MCUT-0.42-AB1DFHMR$]
                                              ^-- dies hier }
    R: BOOLEAN;                 {Resume-Mode}
  END;

  DieBox_SF_Stn_Rec = RECORD     {Daten bei S&F nach W0RLI}
    Send_To: STRING[8];          {Mail wird gesendet an (aus Send-Befehl}
    Send_From: STRING[6];        {Mail wird gesendet von (")}
    Send_Via: STRING[50];        {Verteiler der Mail - kann bei Mailboxen
                                  (Usermail) auch mit Hierachischer Adresse
                                  angegeben werden}
    Hierach: STRING[128];        {Hierachie der Empfnger-Box}
    Titel: STRING[80];           {Titel der Mail}
    Typ: CHAR;                   {'P' oder 'B' - Usermail oder Bulletin
                                  M=MYBBS-Forwarding}
    BID: STRING[15];             {Bulletin-ID der Mail}
    LT: INTEGER;                 {Lifetime der Mail}
    SFPartner: STRING[6];        {1. SF-Partner, von dem diese Mail gekommen
                                  ist. (fr den Autorouter}
    AuswertHeader: BOOLEAN;      {Steht solange auf TRUE, bis die erste
                                  Zeile empfangen wird, die kein S&F-header
                                  ist. (Autorouter)}
    Features: SF_Features;
    status: BYTE;                {Status vom S&F}
    f: Text;                     {File, in das das empfangene File
                                  geschrieben wird}
    FileName: STRING[40];        {Filename, in das geschrieben werden soll}
    F_Get: BOOLEAN;              {Wenn F> empfangen wurde, wird F_Get auf
                                  true gesetzt}
    RX_Date: LONGINT;            {Zeitpunkt, an dem die Mail empfangen wurde
                                  (fr den S&F-Header)}
    RX_From: STRING[6];          {S&F-Partner der letzten Mail dieser Station
                                  (nicht an userer sondern an der gegenseite)}
    SID_OK: BOOLEAN;             {SID ist korrekt empfangen worden (beim
                                  senden)}
    Send_Info: BOOLEAN;
    LT_Angegeben: BOOLEAN;       {TRUE, wenn eine BID angegeben wurde (in
                                  der S-Zeile}
    BIN: BOOLEAN;                {Wurde ein BIN-Teil entdeckt?}
    BIN_Len: LONGINT;            {Lnge des BIN-Teils der Mail}
    BIN_Akt: LONGINT;            {Wieviele Bytes des BIN-Teils wurden
                                  schon empfangen}
    BIN_CRC_Got: WORD;           {BID-Checksumme (empfangen)}
    BIN_CRC: WORD;               {Selbst erreichnete 16 Bit-Checksumme}
    BIN_File: FILE;              {FILE, in dem die BIN-Mails gespeichert
                                  werden (fr BLOCKREAD/BLOCKWRITE)}
    ForwWhat: CHAR;              {Was wird geforwardet (in der jeweiligen
                                  Zeit) - Wird nur fr automatisches
                                  Forward benutzt - sonst ' '
                                  Entweder A (Alles forwarden) oder U
                                  (nur Usermails forwarden)}
    ForwSize: WORD;              {Files bis zu welcher gre werden
                                  geforwardet?}
    TheBox_E_M_Format: BOOLEAN;  {Wird das verkrzte DieBox-Format fr
                                  E und M-Mails verwendet?}
    WhichSID: BYTE;              {Welche SID soll verwendet werden? (aus
                                  BOX.SF)}
    UsermailAbgelehnt: BOOLEAN;  {Wurde eine Usermail abgelehnt?}
    PwdAnswer: STRING[40];        {Antwort-String auf einen Passwort-
                                  geschtzten Forward}
    PW_Gesichert: BOOLEAN;       {TRUE, wenn das S&F Passwort-gesichert
                                  ist}
    BOXSF_Partner: BOOLEAN;      {Ist dieser S&F-Partner ein in BOX.SF
                                  eingetragener S&F-Partner?}
    SchmeissWech: BOOLEAN;       {Die Mail wird nach dem empfang geloescht}
  END;

  tFBB_SF_Mail_Rec = RECORD
    Typ: CHAR;                   {Typ der Mail - 'P' oder 'B'
                                  'M' = MYBBS-Forwarding oder 'A' =ACK-Mail}
    Sender: STRING[6];           {Von wem ist die Mail?}
    Verteiler: STRING[50];       { @ XXX, kann auch incl. hierachischem Pfad
                                  sein!}
    An: STRING[8];               {Call oder Rubrik, an den die Mail geht}
    AnTransfer: STRING[8];       {Rubrik, nachdem das Transfer-File
                                  abgearbeitet wurde}
    Hierach: STRING[128];        {Hierachie der Empfnger-Box}
    BID: STRING[12];             {Bulletin-ID dieser Mail}
    Size: LONGINT;                  {Gre der Mail in Bytes}
    Send: CHAR;                  {Senden '+', Nicht senden '-', warten '='}
    LT: WORD;                    {Lifetime}
    Del: BOOLEAN;                {Mail nach S&F lschen? (Bei Usermails)}
    FileName: STRING[40];        {Filename, in die das File mu (ohne
                                  BOXPATH!!}
    Titel: STRING[80];           {Titel der Mail}
    RX_Date: LONGINT;            {RX_Date als PackTime}
    Send_Info: BOOLEAN;          {Info in [] senden?}
    LT_Angegeben: BOOLEAN;       {True, wenn eine LifeTime angegeben wurde}
    StartOffset: LONGINT;        {Ab welchem Offset soll das File gesendet
                                  werden? (Resume-Mode)}
  END;

  FBB_SF_Stn_Rec = RECORD        {Daten fr S&F nach F6FBB (und erweiterung)}
    MailDat: ARRAY[1..10] OF tFBB_SF_Mail_Rec;    {Informationen der Mails
                                  in einem Vorschlag (max 5 Mails)}
    SFPartner: STRING[6];        {Ebenfalls info fr den Autorouter}
    RXFrom: STRING[6];
    AuswertHeader: BOOLEAN;      {Steht solange auf TRUE, bis die erste
                                  Zeile empfangen wird, die kein S&F-header
                                  ist.}
    TPKMode:BOOLEAN;             {4iad,mar 97:
                                    TPK-Modus, anfordern bestimmter Bulletins
                                    per S&F. Die Daten stehen in
                                    Maildat[1]. 
                                    AnzPropsal muss auf 1 stehen.
                                    Nach dem Empfang der Nachricht wird die
                                    Checkliste nach der naechsten Nr befragt.
                                    Ist nichts mehr vorhanden, disconnecten.
                                  }  

    TPKListPos:LONGINT;           {Im TPK-Modus steht hier die Nummer der mail
                                   in der Checkliste.}                    
    AnzProposal: BYTE;           {Wieviele Zeilen wurden vor dem F>
                                  gesendet?}
    
    Features: SF_Features;
    ChckSum: BYTE;               {Checksumme}
    Chck16: WORD;                {16-Bit-Checksumme}
    status: BYTE;                {Status vom S&F}
    f: Text;                     {File, in das das empfangene File
                                  geschrieben wird}
    Akt: BYTE;                   {Aktuell empfangene Mail}
    Set2Null: BOOLEAN;           {Nach S&F AnzProposal auf 0 gesetzt?}
    Tmp: STRING[5];              {Temporrer String}

    {Binres S&F nach F6FBB}
    Binary: FILE;                {Untypisiertes File, in das erstmal die
                                  Binary-Sachen 'rein geschrieben werden}
    Temp: STRING;                {Temporr eine Zeile speichern (gepackt)}
    DelOK: BOOLEAN;              {Nach dem ersten Proposal (nach der FS ++
                                  Zeile) ein Delay von 1s einfgen}
    ForwWhat: CHAR;              {Was wird geforwardet (in der jeweiligen
                                  Zeit) - Wird nur fr automatisches
                                  Forward benutzt - sonst ' '
                                  Entweder A (Alles forwarden) oder U
                                  (nur Usermails forwarden)}
    ForwSize: WORD;              {Files bis zu welcher gre werden
                                  geforwardet?}
    PwdAnswer: STRING[40];        {Antwort-String auf einen Passwort-
                                  geschtzten Forward}
    PW_Gesichert: BOOLEAN;       {TRUE, wenn das S&F Passwort-gesichert
                                  ist}
    BOXSF_Partner: BOOLEAN;      {Ist dieser S&F-Partner ein in BOX.SF
                                  eingetragener S&F-Partner?}
    SchmeissWech: BOOLEAN;       {Die Mail wird nach dem empfang geloescht}
    
  END;

(**************************************************************************)
(*                      BID-Hashing Record (iad,nov 95)                   *)
(**************************************************************************)

  TBIDHashRec  =   RECORD
     rubrik:STRING[8];
     idx_file_pos:WORD;
     check_file_pos:LONGINT;
  END;


VAR OwnBoxData: ^T_OwnBoxData;  {Daten der eigenen Box}
    LastTime: LONGINT;          {Zeitpunkt des Abschickens der letzten Mail
                                 (UnixTime)}
    AnzPerSec: BYTE;            {Anzahl Mails pro Sekunde}
    DirInfo: ARRAY[1..500] OF ^R_DirInfo;     {Informationen fr den DIR-
                                 Befehl - bis zu 500 Rubriken (User- und
                                 Info-Rubriken zusammen}
    AnzRub: WORD;               {Anzahl vorhandener, eingelesener Rubriken}
    SID: STRING[25];            {SID der Mailbox}
    i: BYTE;

    Reject_Box_Root: pReject_Box;       {Abbild von REJECT.BOX im Speicher
                                 (zum schnelleren Zugriff)}
    BoxSFRoot: pBoxSF;          {BOX.SF im Speicher}

    BidHash,UserHash:PFhash;

    OnlineHelp: pOnlineHelp;

IMPLEMENTATION


BEGIN
  Reject_Box_Root := NIL;
  BoxSFRoot := NIL;
END.


{

Hier will ich mal jeden von mir benutzten Status kommentieren, damit ich
die bersicht wieder bekomme... :-(

  50: Normale Mailbox-Befehl - Standard-Status bei Mailbox-Idle
  51: DieBox-S&F
  52: F6FBB-S&F
  59: Warten auf Passwort-Eingabe
  60: S&F-Connect wurde gemacht, aber noch nicht abgeschlossen
  80: Check-Ausgabe luft
  81: List-Ausgabe luft
}
