unit sockets;

{ -------------------------------------------------------------------
    Unit for easy socket supprt under Linux FPK pascal.
    Copyright (C) 1997	Michael Van Canneyt

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 1, or (at your option)
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  
  Change Log
  ----------

  Started by Michael Van Canneyt, June 1997 
  (michael@tfdec1.fys.kuleuven.ac.be)
  
   Current version 0.2.1

   Date       Version     Who       Comments
   June/1997  0.1         Michael   Initial implementation
   07/15/97   0.2         Bazsi     fixed address-type records, added
                                    changelog
   07/15/96   0.3         Michael   Adjusted size of records.
                                    Fixed error in Connect call.
                                    implemented more calls.
                                    started untyped-file support.
                                    put GPL in.
   12/13/97		  Hans-Peter Zorn  adopted to new rtl. 

  Todo:
    remove declaration of TextRec from here, and put it into one of the API
    units
  ------------------------------------------------------------------- }

Interface

Uses linux;

Const
 { Socket Types }
  SOCK_STREAM     = 1;               { stream (connection) socket   }
  SOCK_DGRAM      = 2;               { datagram (conn.less) socket  }
  SOCK_RAW        = 3;               { raw socket                   }
  SOCK_RDM        = 4;               { reliably-delivered message   }
  SOCK_SEQPACKET  = 5;               { sequential packet socket     }
  SOCK_PACKET     =10;

  { Adress families }
  AF_UNSPEC       = 0;
  AF_UNIX         = 1;      { Unix domain sockets          }
  AF_INET         = 2;      { Internet IP Protocol         }
  AF_AX25         = 3;      { Amateur Radio AX.25          }
  AF_IPX          = 4;      { Novell IPX                   }
  AF_APPLETALK    = 5;      { Appletalk DDP                }
  AF_NETROM       = 6;      { Amateur radio NetROM         }
  AF_BRIDGE       = 7;       { Multiprotocol bridge         }
  AF_AAL5         = 8;       { Reserved for Werner's ATM    }
  AF_X25          = 9;       { Reserved for X.25 project    }
  AF_INET6        = 10;      { IP version 6                 }
  AF_MAX          = 12;
 
 {  Protocol Families }
  
  PF_UNSPEC       = AF_UNSPEC;
  PF_UNIX         = AF_UNIX;
  PF_INET         = AF_INET;
  PF_AX25         = AF_AX25;
  PF_IPX          = AF_IPX;
  PF_APPLETALK    = AF_APPLETALK;
  PF_NETROM       = AF_NETROM;
  PF_BRIDGE       = AF_BRIDGE;
  PF_AAL5         = AF_AAL5;
  PF_X25          = AF_X25;
  PF_INET6        = AF_INET6;

  PF_MAX          = AF_MAX;   

Type
  {$packrecords 1}
  TSockAddr = Record
    family : word;  { was byte, fixed }
    data   : array [0..13] of char;
    end;

  TUnixSockAddr = Record
    family : word; { was byte, fixed }
    path : array[0..108] of char;
    end;
  
  TInetSockAddr = Record
    family : Word;
    port   : Word;
    addr   : Cardinal;
    pad    : array [1..8] of byte; { to get to the size of sockaddr... }
    end;

  TSockArray = Array[1..2] of Longint;
  
Var SocketError : Longint; 

Function  Socket (Domain,SocketType,Protocol : Longint) : Longint;
Procedure Bind (Sock : Longint; Var Addr; AddrLen : Longint);
Procedure Listen (Sock : Longint; MaxConnect : Longint);

Procedure Accept (Sock : Longint;  Var Addr; Var Addrlen : Longint;Var SockIn,SockOut : Text);
Procedure Connect (Sock : Longint; Var Addr; Addrlen : Longint;Var SockIn,SockOut : Text);
{ 
  untyped file socket support is not yet ready. System unit needs changing ! 
}
Procedure Accept (Sock : Longint;  Var Addr; Var Addrlen : Longint;Var SockIn,SockOut : File);
Procedure Connect (Sock : Longint; Var Addr; Addrlen : Longint;Var SockIn,SockOut : File);

Procedure Shutdown (Sock : Longint; How:  Longint);
Procedure GetSocketName (Sock : Longint; Var Addr; Var Addrlen : Longint);
Procedure GetPeerName (Sock : Longint; Var Addr; Var Addrlen : Longint);
Procedure SetSocketOptions (Sock,Level,OptName : Longint; Var OptVal; 
                            optlen : longint);
Procedure GetSocketOptions (Sock,Level,OptName : Longint; Var OptVal; 
                            var optlen : longint);

Procedure SocketPair (Domain,SocketType,Protocol : Longint; var Pair : TSockArray);

Implementation

Const
  {
    Arguments to the Linux Kernel system call for sockets. All
    Socket Connected calls go through the same system call,
    with an extra argument to determine what action to take.
  }
  Socket_SYS_SOCKET      = 1;
  Socket_SYS_BIND        = 2;
  Socket_SYS_CONNECT     = 3;
  Socket_SYS_LISTEN      = 4;
  Socket_SYS_ACCEPT      = 5;
  Socket_SYS_GETSOCKNAME = 6;
  Socket_SYS_GETPEERNAME = 7;
  Socket_SYS_SOCKETPAIR  = 8;
  Socket_SYS_SEND        = 9;
  Socket_SYS_RECV        = 10;
  Socket_SYS_SENDTO      = 11;
  Socket_SYS_RECVFROM    = 12;
  Socket_SYS_SHUTDOWN    = 13;
  Socket_SYS_SETSOCKOPT  = 14;
  Socket_SYS_GETSOCKOPT  = 15;
  Socket_SYS_SENDMSG     = 16;
  Socket_SYS_RECVMSG     = 17;
  
  { Two constants to determine whether part of soket is for in or output }
  S_IN = 0;
  S_OUT = 1;


{ The structure below should be taken from one of the API units... }
Type
  TextBuf = array[0..127] of char;

  TextRec = record
    handle : word;
    mode : word;
    bufSize : word;
    _private : word;
    bufpos : word;
    bufend : word;
    bufptr : ^textbuf;
    openfunc : pointer;
    inoutfunc : pointer;
    flushfunc : pointer;
    closefunc : pointer;
    userdata : array[1..16] of byte;
    name : array[0..255] of char;
    buffer : textbuf;
    End;       

Function socket (Domain,SocketType,Protocol : Longint) : Longint;

var Temp : syscallregs;
    args : array[1..3] of longint;
    
begin
  SocketError:=0;
  args[1]:=Domain;
  args[2]:=SocketType;
  args[3]:=Protocol;
  Temp.reg2:=Socket_Sys_Socket;
  temp.reg3:=Longint(@args);
  Socket:=Syscall (syscall_nr_socketcall,temp);
  If Socket<0 then
    SocketError:=Errno; 
end;

Procedure Bind (Sock : Longint; Var Addr; AddrLen : Longint);

Var Temp : syscallregs;
    args : array[1..3] of longint;

begin
   SocketError:=0;
   args[1]:=Sock;
   args[2]:=Longint(@Addr);
   args[3]:=Addrlen;
   Temp.reg2:=Socket_Sys_Bind;
   temp.reg3:=Longint(@args);
   if Syscall (syscall_nr_socketcall,temp)<0 then
      SocketError:=Errno;
end;

Procedure Listen (Sock : Longint; MaxConnect : Longint);

Var temp : syscallregs;
    args : array[1..2] of Longint;

begin
  SocketError:=0;
  args[1]:=Sock;
  args[2]:=MaxConnect;
  Temp.reg2:=Socket_Sys_Listen;
  Temp.reg3:=Longint(@args);
  If Syscall (syscall_nr_socketcall,temp)<0 then
    SocketError:=Errno;
end;

{ Procedures for reading/writing sockets}

Procedure OpenSock (var F : Text);

begin
{$ifdef debug}
  write ('OpenSock called for fd ',textrec(f).handle,' with mode ');
  if textrec(f).mode=fmoutput then writeln ('Output') else writeln ('input');
{$endif}
  if (textrec(f).mode=fmoutput) then
    begin
    if textrec(f).userdata[1]<>S_OUT then 
      textrec(f).mode:=fmclosed;
    end
  else if textrec(f).mode=fminput then
     begin
     if textrec(f).userdata[1]<>S_IN then textrec(f).mode:=fmclosed
     end
  else
     textrec(f).mode:=fmclosed;
end; 

Procedure IOSock (var F : text);

begin
{$ifdef debug}
  write ('IOSock called for fd ',textrec(f).handle,' with mode ');
  if textrec(f).mode=fmoutput then writeln ('Output') else 
     if textrec(f).mode=fminput then writeln ('input') else writeln('Closed');
{$endif}
  If textrec(f).mode=fmoutput then
    fdwrite (textrec(f).handle,pchar(textrec(f).bufptr),textrec(f).bufpos)
  else if textrec(f).mode=fminput then
    textrec(f).bufend:=
    fdread(textrec(f).handle,pchar(textrec(f).bufptr),textrec(f).bufsize);
  textrec(f).bufpos:=0;
end;

Procedure FlushSock ( var F : Text);

begin
{$ifdef debug}
  writeln ('FlushSock called with FD ',textrec(f).handle,'.');
{$endif}
  if (textrec(f).mode=fmoutput) and (textrec(f).bufpos<>0) then IOSock (f);
  textrec(f).bufpos:=0;
end;
      
      
Procedure CloseSock ( var F : text);
     
begin
{$ifdef debug}
   writeln ('CloseSock called with FD ',textrec(f).handle,'.');
{$endif}
   textrec(f).mode := fmclosed;
   fdclose (textrec(f).handle);
end;

{ Auxiliary procedure for Accept Connect }

Procedure AssignPair (SHandle : Longint; Var SockIn,SockOut : Text);
{
 Set up two Pascal Text file descriptors for reading and writing)
}
begin
  { First the reading part.}
  Textrec(SockIn).Handle:=SHandle;
  Textrec(SockIn).mode:=fmclosed;
  Textrec(Sockin).userdata[1]:=S_IN;
  TextRec(SockIn).Name[0]:=#0;
  Textrec(SockIn).BufSize:=SizeOf(TextBuf);
  TextRec(SockIn).BufPtr:=@TextRec(Sockin).Buffer;
  TextRec(SockIn).OpenFunc  := @OpenSock;
  TextRec(SockIn).InOutFunc := @IOSock;
  TextRec(SockIn).FlushFunc := @FlushSock;
  TextRec(SockIn).CloseFunc := @CloseSock;
  { Now the writing part. }
  Textrec(SockOut).Handle:=SHandle;
  Textrec(SockOut).mode:=fmclosed;
  Textrec(SockOut).userdata[1]:=S_OUT;
  TextRec(SockOut).Name[0]:=#0;
  Textrec(SockOut).BufSize:=SizeOf(TextBuf);
  TextRec(SockOut).BufPtr:=@TextRec(SockOut).Buffer;
  TextRec(SockOut).OpenFunc  := @OpenSock;
  TextRec(SockOut).InOutFunc := @IOSock;
  TextRec(SockOut).FlushFunc := @FlushSock;
  TextRec(SockOut).CloseFunc := @CloseSock;
end;

Procedure Accept (Sock : Longint; Var Addr; Var Addrlen : Longint;Var SockIn,SockOut : Text);

Var S : Longint;
    Temp : Syscallregs;
    args : array[1..3] of Longint;

begin
  SocketError:=0;
  args[1]:=Sock;
  args[2]:=Longint(@Addr);
  args[3]:=Longint(@Addrlen);
  temp.reg2:=Socket_Sys_accept;
  temp.reg3:=Longint(@args);
  S:=Syscall(syscall_nr_socketcall,temp);
  if s<0 then 
    begin
    SocketError:=Errno;
    exit
    end;
  AssignPair(S,SockIn,SockOut);
end;

Procedure Connect (Sock : Longint; Var Addr; Addrlen : Longint;Var SockIn,SockOut : Text);

Var S : Longint;
    Temp : Syscallregs;
    args : array[1..3] of Longint;

begin
  SocketError:=0;
  args[1]:=Sock;
  args[2]:=Longint(@Addr);
  args[3]:=Addrlen;
  temp.reg2:=Socket_Sys_Connect;
  temp.reg3:=Longint(@args);
  S:=Syscall(syscall_nr_socketcall,temp);
  if s<0 then
    begin 
    SocketError:=Errno;
    exit
    end;
  AssignPair (Sock,SockIn,SockOut);
end;

Procedure Accept (Sock : Longint;  Var Addr; Var Addrlen : Longint;Var SockIn,SockOut : File);

begin
  { We cannot do the cast without problems. Yet. }
{  TextRec(SockIn).Handle:=Sock;
  TextRec(SockOut).Handle:=Sock;
  TextRec(SockIn).Mode:=fmclosed;
  TextRec(SockOut).Mode:=fmclosed;}
end;

Procedure Connect (Sock : Longint; Var Addr; Addrlen : Longint;Var SockIn,SockOut : File);

begin
  { We cannot do the cast without problems. Yet. }
{  TextRec(SockIn).Handle:=Sock;
  TextRec(SockOut).Handle:=Sock;
  TextRec(SockIn).Mode:=fmclosed;
  TextRec(SockOut).Mode:=fmclosed;}
end;

Procedure Shutdown (Sock : Longint; How:  Longint);

var args : array[1..2] of longint;
    temp : syscallregs;
    s : longint;
    
begin
  SocketError:=0;
  args[1]:=Sock;
  args[2]:=How;
  temp.reg2:=Socket_Sys_Shutdown;
  temp.reg3:=Longint(@args);
  if Syscall(syscall_nr_socketcall,temp)<>0 then
     SocketError:=Errno; 
end;

Procedure GetSocketName (Sock : Longint; Var Addr; Var Addrlen : Longint);

var args : array[1..3] of longint;
    temp : syscallregs;
    s : longint;
    
begin
  SocketError:=0;
  args[1]:=Sock;
  args[2]:=Longint(@Addr);
  args[3]:=Longint(@Addrlen);
  temp.reg2:=Socket_Sys_GetSockName;
  temp.reg3:=Longint(@args);
  if Syscall(syscall_nr_socketcall,temp)<>0 then
     SocketError:=Errno; 
end;

Procedure GetPeerName (Sock : Longint; Var Addr; Var Addrlen : Longint);

var args : array[1..3] of longint;
    temp : syscallregs;
    s : longint;
    
begin
  SocketError:=0;
  args[1]:=Sock;
  args[2]:=Longint(@Addr);
  args[3]:=Longint(@Addrlen);
  temp.reg2:=Socket_Sys_GetPeerName;
  temp.reg3:=Longint(@args);
  if Syscall(syscall_nr_socketcall,temp)<>0 then
     SocketError:=Errno; 
end;

Procedure SetSocketOptions (Sock,Level,OptName : Longint; Var OptVal; 
                            optlen : longint);

var args : array[1..5] of longint;
    temp : syscallregs;
    s : longint;
    
begin
  SocketError:=0;
  args[1]:=Sock;
  args[2]:=Level;
  args[3]:=OptName;
  args[4]:=Longint(@optval);
  args[5]:=optlen;
  temp.reg2:=Socket_Sys_SetSockOpt;
  temp.reg3:=Longint(@args);
  if Syscall(syscall_nr_socketcall,temp)<>0 then
     SocketError:=Errno; 
end;

Procedure GetSocketOptions (Sock,Level,OptName : Longint; Var OptVal; 
                            var optlen : longint);

var args : array[1..5] of longint;
    temp : syscallregs;
    s : longint;
    
begin
  SocketError:=0;
  args[1]:=Sock;
  args[2]:=Level;
  args[3]:=OptName;
  args[4]:=Longint(@optval);
  args[5]:=Longint(@optlen);
  temp.reg2:=Socket_Sys_GetSockOpt;
  temp.reg3:=Longint(@args);
  if Syscall(syscall_nr_socketcall,temp)<>0 then
     SocketError:=Errno; 
end;

Procedure SocketPair (Domain,SocketType,Protocol : Longint; var Pair : TSockArray);

var args : array[1..4] of longint;
    temp : syscallregs;
    s : longint;
    
begin
  SocketError:=0;
  args[1]:=Domain;
  args[2]:=SocketType;
  args[3]:=Protocol;
  args[4]:=Longint(@Pair);
  temp.reg2:=Socket_Sys_SocketPair;
  temp.reg3:=Longint(@args);
  if Syscall(syscall_nr_socketcall,temp)<>0 then
     SocketError:=Errno; 
end;

end.

