(****************************************************************************)
(* Ansteuerung von AX25/V24 Treibern.                                       *)
(* Unter anderem                                                            *)
(*   * Jochens V24-Unit                                                     *)
(*   * TFPC*                                                                *)
(*   * DRSI (TNCTSR und TFPCX /dr)                                          *)
(*   * MBBIOS u..                                                          *)
(*                                                                          *)
(* Die Unit darf nicht als Overlay deklariert werden.                       *)
(*                                                                          *)
(* Diese Unit ist Teil des MCUT-Sourcecodes, alle Rechte liegen bei         *)
(* DG4IAD und DG6VJ                                                         *)
(*                                                                          *)
(****************************************************************************)

UNIT v24DRV;

INTERFACE

FUNCTION V24Open(comport,irq,adresse,typ:WORD;baud:LONGINT):WORD;
PROCEDURE V24Close;
FUNCTION  V24Read:CHAR;
PROCEDURE V24Write(c:CHAR);
FUNCTION  V24Install(comport:BYTE;adresse:WORD;TYP:BYTE):BOOLEAN;
FUNCTION  V24Dataavail:BOOLEAN;
IMPLEMENTATION

USES VJ_V24,VJ_TFPCX;

CONST
   NOT_ATTACHED=0;
   DRV_VJ=1;
   DRV_TFPC=2;
   DRV_DRSI=3;
   DRV_INT14=4;

CONST drv_typ:BYTE=0;



FUNCTION V24Open(comport,irq,adresse,typ:WORD;baud:LONGINT):WORD;
BEGIN
  V24Open:=0;
  IF DRV_TYP<>NOT_ATTACHED THEN exit;
  CASE typ OF
    DRV_VJ:
       BEGIN
         IF IRQ <> 0 THEN COMirq[comport]:=IRQ;
         IF adresse <> 0 THEN COMadr[comport]:=adresse;
         IF NOT V24_PortAvailable(COMadr[comport]) THEN Exit;
         V24Open:=V24_Init(baud,8,'N',1,comport);
         drv_typ:=DRV_VJ;
       END;
    DRV_TFPC:
       BEGIN
         drv_typ:=DRV_TFPC;
       END;
   END;
END;

PROCEDURE V24Close;
BEGIN
  IF drv_TYP=NOT_ATTACHED THEN exit;
  CASE drv_typ OF
    DRV_VJ:v24_deinit;
  END;
  DRV_TYP:=NOT_ATTACHED;
END;

FUNCTION  V24Read:CHAR;
BEGIN
  CASE DRV_TYP OF
    DRV_VJ:V24Read:=v24_GetChar;
    DRV_TFPC:V24Read:=TFPCX_GetChar;
  END;
END;

PROCEDURE V24Write;
BEGIN
  CASE DRV_TYP OF
    DRV_VJ:V24_SendChar(c);
    DRV_TFPC:TFPCX_SendChar(c);
  END;
END;

FUNCTION  V24DataAvail:BOOLEAN;
BEGIN
  CASE DRV_TYP OF
    DRV_VJ:V24DataAvail:=V24_DataAvailable;
    DRV_TFPC:V24DataAvail:=TFPCX_DataAvailable;
  END;
END;


FUNCTION V24Install;
BEGIN
  V24Install:=FALSE;
  CASE typ OF
    DRV_VJ:
       BEGIN
         IF adresse <> 0 THEN COMadr[comport]:=adresse;
         V24Install:=V24_PortAvailable(COMadr[comport]);
       END;
    DRV_TFPC:
       BEGIN
         V24Install:=TFPCX_Installed;
       END;
   END;
END;


END.